@@ -5,6 +5,7 @@ use Dancer::Plugin::DBIC 'schema';
 
 use App::Netdisco::Util::Device qw/get_device is_discoverable/;
 use App::Netdisco::Util::DNS ':all';
+use App::Netdisco::JobQueue qw/jq_queued jq_insert/;
 use NetAddr::IP::Lite ':lower';
 use List::MoreUtils ();
 use Encode;
@@ -168,7 +169,7 @@ sub store_device {
   $device->model(  Encode::decode('UTF-8', $snmp->model)  );
   $device->serial( Encode::decode('UTF-8', $snmp->serial) );
 
-  $device->snmp_class( $snmp->device_type );
+  $device->snmp_class( $snmp->class );
   $device->last_discover(\'now()');
 
   schema('netdisco')->txn_do(sub {
@@ -215,9 +216,20 @@ sub store_interfaces {
   # clear the cached uptime and get a new one
   my $dev_uptime = $snmp->load_uptime;
 
-  # used to track whether we've wrapped the device uptime
+  # used to track how many times the device uptime wrapped
   my $dev_uptime_wrapped = 0;
 
+  # use SNMP-FRAMEWORK-MIB::snmpEngineTime if available to
+  # fix device uptime if wrapped
+  if (defined $snmp->snmpEngineTime) {
+      $dev_uptime_wrapped = int( $snmp->snmpEngineTime * 100 / 2**32 );
+      if ($dev_uptime_wrapped > 0) {
+          info sprintf ' [%s] interface - device uptime wrapped %d times - correcting',
+            $device->ip, $dev_uptime_wrapped;
+          $device->uptime( $dev_uptime + $dev_uptime_wrapped * 2**32 );
+      }
+  }
+
   # build device interfaces suitable for DBIC
   my %interfaces;
   foreach my $entry (keys %$interfaces) {
@@ -263,7 +275,7 @@ sub store_interfaces {
                   debug sprintf
                     ' [%s] interfaces - correcting LastChange for %s, assuming sysUptime wrap',
                     $device->ip, $port;
-                  $lc += 2**32;
+                  $lc += $dev_uptime_wrapped * 2**32;
               }
           }
       }
@@ -651,13 +663,13 @@ sub store_neighbors {
   my $c_cap      = $snmp->c_cap;
 
   foreach my $entry (List::MoreUtils::uniq( (keys %$c_ip), (keys %$c_cap) )) {
-      my $port = $interfaces->{ $c_if->{$entry} };
-      if (!defined $port) {
+      if (!defined $c_if->{$entry} or !defined $interfaces->{ $c_if->{$entry} }) {
           debug sprintf ' [%s] neigh - port for IID:%s not resolved, skipping',
             $device->ip, $entry;
           next;
       }
 
+      my $port = $interfaces->{ $c_if->{$entry} };
       my $portrow = schema('netdisco')->resultset('DevicePort')
           ->single({ip => $device->ip, port => $port});
 
@@ -900,20 +912,12 @@ sub discover_new_neighbors {
           next;
       }
 
-      # Don't queued if job already exists
-      my $is_queued = schema('netdisco')->resultset('Admin')->search(
-          {   device => $ip,
+      # Don't queue if job already exists
+      if (List::MoreUtils::none {$_ eq $ip} jq_queued('discover')) {
+          jq_insert({
+              device => $ip,
               action => 'discover',
-              status => { -like => 'queued%' },
-          }
-      )->single;
-      unless ($is_queued) {
-          schema('netdisco')->resultset('Admin')->create(
-              {   device => $ip,
-                  action => 'discover',
-                  status => 'queued',
-              }
-          );
+          });
       }
   }
 }
